CheckpointManager

Struct CheckpointManager 

pub struct CheckpointManager<R: Runtime> { /* private fields */ }
Manages checkpoints for adjoint sensitivity computation.

During forward integration, checkpoints are stored at strategic times. During backward adjoint integration, the forward solution is reconstructed by reintegrating between checkpoints as needed.

Implementations

Source

impl<R: Runtime> CheckpointManager<R>

pub fn new( n_checkpoints: usize, strategy: CheckpointStrategy, t_span: [f64; 2], ) -> Self

Create a new checkpoint manager.

Arguments
  • n_checkpoints - Number of checkpoints to store (excluding t0 and T)
  • strategy - Strategy for placing checkpoints
  • t_span - Integration interval [t0, T]
pub fn checkpoint_times(&self) -> &[f64]

Get the planned checkpoint times.

pub fn add_checkpoint(&mut self, t: f64, y: Tensor<R>)

Add a checkpoint at the given time and state.

pub fn checkpoints(&self) -> &[Checkpoint<R>]

Get all stored checkpoints.

pub fn len(&self) -> usize

Get the number of stored checkpoints.

pub fn is_empty(&self) -> bool

Check if no checkpoints are stored.

pub fn find_interval(&self, t: f64) -> Option<(usize, usize)>

Find the checkpoint interval containing time t.

Returns (checkpoint_before, checkpoint_after) where checkpoint_before.t <= t <= checkpoint_after.t

For backward adjoint integration, we need the checkpoint just before the current time to start reintegrating from.

pub fn get(&self, index: usize) -> Option<&Checkpoint<R>>

Get checkpoint at index.

pub fn iter_reverse(&self) -> impl Iterator<Item = &Checkpoint<R>>

Iterate checkpoints in reverse order (for backward pass).

pub fn should_checkpoint(&self, t: f64, tol: f64) -> bool

Check if a time is close to a planned checkpoint time.